from functools import wraps
from http import HTTPStatus


class UnauthorizedError(Exception):
    """
    Exception raised for unauthorized access errors.

    Attributes:
        message (str): Explanation of the error.
        status_code (HTTPStatus): HTTP status code for the error.
    """

    def __init__(self, message):
        self.message = message
        self.status_code = HTTPStatus.UNAUTHORIZED


class NotFoundError(Exception):
    """
    Exception raised for not found errors.

    Attributes:
        message (str): Explanation of the error.
        status_code (HTTPStatus): HTTP status code for the error.
    """

    def __init__(self, message):
        self.message = message
        self.status_code = HTTPStatus.NOT_FOUND


class BadRequestError(Exception):
    """
    Exception raised for bad request errors.

    Attributes:
        message (str): Explanation of the error.
        status_code (HTTPStatus): HTTP status code for the error.
    """

    def __init__(self, message):
        self.message = message
        self.status_code = HTTPStatus.BAD_REQUEST


def handle_exceptions(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except UnauthorizedError as e:
            return {"msg": e.message}, e.status_code
        except NotFoundError as e:
            return {"msg": e.message}, e.status_code
        except BadRequestError as e:
            return {"msg": e.message}, e.status_code
        except Exception as e:
            return {
                "msg": "An unexpected error occurred: " + str(e)
            }, HTTPStatus.INTERNAL_SERVER_ERROR

    return wrapper



"""Common functionality for the vantage6 server and algorithm store."""

from vantage6.common.globals import DEFAULT_API_PATH

# make sure the version is available
from vantage6.backend.common._version import __version__  # noqa: F401


def get_server_url(
    config: dict, server_url_from_request: str | None = None
) -> str | None:
    """ "
    Get the server url from the request data, or from the configuration if it is
    not present in the request.

    Parameters
    ----------
    config : dict
        Server configuration
    server_url_from_request : str | None
        Server url from the request data.

    Returns
    -------
    str | None
        The server url
    """
    if server_url_from_request:
        return server_url_from_request
    server_url = config.get("server_url")
    # make sure that the server url ends with the api path
    api_path = config.get("api_path", DEFAULT_API_PATH)
    if server_url and not server_url.endswith(api_path):
        server_url = server_url + api_path
    return server_url



from typing import List
from marshmallow import Schema, fields, validates, ValidationError
from marshmallow.validate import Length, Range

MAX_LEN_NAME = 128
MAX_LEN_STR_LONG = 1024


def validate_name(name: str) -> None:
    """
    Validate a name field in the request input.

    Parameters
    ----------
    name : str
        Name to validate.

    Raises
    ------
    ValidationError
        If the name is empty, too long or numerical
    """
    if not len(name):
        raise ValidationError("Name cannot be empty")
    if name.isnumeric():
        raise ValidationError("Name cannot a number")
    if len(name) > MAX_LEN_NAME:
        raise ValidationError(f"Name cannot be longer than {MAX_LEN_NAME} characters")


class _NameValidationSchema(Schema):
    """Schema for validating POST requests with a name field."""

    name = fields.String(required=True)

    @validates("name")
    def validate_name(self, name: str):
        """
        Validate the name in the input.

        Parameters
        ----------
        name : str
            Name to validate.

        Raises
        ------
        ValidationError
            If the name is empty, too long or numerical
        """
        validate_name(name)


class RoleInputSchema(_NameValidationSchema):
    """Schema for validating input for creating a role."""

    def __init__(self, default_roles: List[str]):
        super().__init__()
        self.default_roles = default_roles

    description = fields.String(validate=Length(max=MAX_LEN_STR_LONG))
    rules = fields.List(fields.Integer(validate=Range(min=1)), required=True)

    @validates("name")
    def validate_name(self, name: str):
        """
        Validate that role name is not one of the default roles.

        Parameters
        ----------
        name : str
            Role name to validate.

        Raises
        ------
        ValidationError
            If the role name is one of the default roles.
        """
        if name in self.default_roles:
            raise ValidationError("Role name cannot be one of the default roles")


class ServerRoleInputSchema(RoleInputSchema):
    """Schema for validating input for creating a server role."""

    organization_id = fields.Integer(validate=Range(min=1))



# DB connection session. This is used by the iPython shell (from server import
# session). Flask requests obtain their session from `g.session` which is
# initialized on `pre_request`.
session = None



from http import HTTPStatus
from flask import request
from flask_mail import Mail
from flask_restful import Resource, Api

from vantage6.backend.common.permission import PermissionManagerBase
from vantage6.backend.common.resource.output_schema import BaseHATEOASModelSchema
from vantage6.backend.common.resource.pagination import Page


class BaseServicesResources(Resource):
    """
    Flask resource base class

    Attributes
    ----------
    api : Api
        Api instance
    config: dict
        Configuration dictionary
    permissions : PermissionManagerBase
        Instance of class that manages permissions
    mail : Mail
        Flask Mail instance
    """

    def __init__(
        self, api: Api, config: dict, permissions: PermissionManagerBase, mail: Mail
    ):
        self.api = api
        self.config = config
        self.permissions = permissions
        self.mail = mail

    @staticmethod
    def is_included(field) -> bool:
        """
        Check that a `field` is included in the request argument context.

        Parameters
        ----------
        field : str
            Name of the field to check

        Returns
        -------
        bool
            True if the field is included, False otherwise
        """
        # The logic below intends to find 'x' both in 'include=y&include=x' and
        # 'include=x,y'.
        return field in [
            val for item in request.args.getlist("include") for val in item.split(",")
        ]

    def dump(self, page: Page, schema: BaseHATEOASModelSchema) -> dict:
        """
        Dump based on the request context (to paginate or not)

        Parameters
        ----------
        page : Page
            Page object to dump
        schema : BaseHATEOASModelSchema
            Schema to use for dumping

        Returns
        -------
        dict
            Dumped page
        """
        return schema.meta_dump(page)

    def response(self, page: Page, schema: BaseHATEOASModelSchema):
        """
        Prepare a valid HTTP OK response from a page object

        Parameters
        ----------
        page : Page
            Page object to dump
        schema : BaseHATEOASModelSchema
            Schema to use for dumping

        Returns
        -------
        tuple
            Tuple of (dumped page, HTTPStatus.OK, headers of the page)
        """
        return self.dump(page, schema), HTTPStatus.OK, page.headers
